/*
 * lem2zip
 * Copyright (C) 2007-2008 Carl Reinke
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "lem2zip.h"
#include "compress.h"
#include "decompress.h"

#include <assert.h>
#include <getopt.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define L2Z_COMPRESS false
#define L2Z_DECOMPRESS true

/*
void usage( void );
void version( void );
void help( void );
*/

/*
const char *prog_name = "lem2zip",
	       *prog_ver = "0.2.0",
	       *prog_date = "2008 Sept 14";
*/

int lem2zip(const char* infile, bool decompress)
{
	FILE *fi = fopen(infile, "rb"),
	     *fo = NULL;

    if (fi == NULL) return EXIT_FAILURE;
	
	char temp_name[] = "lem2zip-XXXXXX";
	if ((fo = fopen(mktemp(temp_name), "wb")) == NULL) {
		fprintf(stderr, "lem2zip: could not create temporary file\n");
		return EXIT_FAILURE;
	}

	static Uint8 d_buffer[CHUNK_SIZE]; // decompressed chunk
	Uint32 d_buffer_size;
	Uint32 d_size, d_size_test = 0;
	
	if (decompress) {
		
		if (!read_header(fi, &d_size)) {
			fprintf(stderr, "lem2zip: %s: not in lem2zip format\n", infile);
			unlink(temp_name);
		    return EXIT_FAILURE;
		}
		
		while (!feof(fi)) {
			chunk_type chunk = { 0 };
			
			if (!read_chunk(fi, &chunk)) {
				free(chunk.ref);
				free(chunk.data);
				break;
			}
			
			if (!decompress_chunk(&chunk, d_buffer, &d_buffer_size)) {
				fprintf(stderr, "lem2zip: %s: invalid compressed data\n", infile);
				unlink(temp_name);
			    return EXIT_FAILURE;

			}
			
			d_size_test += d_buffer_size;
			if (fwrite(d_buffer, 1, d_buffer_size, fo) == 0) {
				fprintf(stderr, "lem2zip: %s: disk write failed\n", temp_name);
				unlink(temp_name);
			    return EXIT_FAILURE;
			}
			
			free(chunk.ref);
			free(chunk.data);
			
			if (chunk.flags)
				break;
		}
		
		if (d_size_test != d_size) {
			fprintf(stderr, "lem2zip: %s: unexpected end of file\n", infile);
			unlink(temp_name);
		    return EXIT_FAILURE;
		} else if (!feof(fi)) {
			int temp = ftell(fi);
			fseek(fi, 0, SEEK_END);
			if (temp - ftell(fi) > 0)
				fprintf(stderr, "lem2zip: %s: decompression okay, trailing garbage ignored\n", infile);
		}
		
	} else {
		fseek(fi, 0, SEEK_END);
		d_size = ftell(fi);
		fseek(fi, 0, SEEK_SET);
		
		write_header(fo, d_size);
		
		while (!feof(fi)) {
			chunk_type chunk = { 0 };
			
			d_buffer_size = d_size > CHUNK_SIZE ? CHUNK_SIZE : d_size;
			
			if (fread(d_buffer, d_buffer_size, 1, fi) == 0) {
				fprintf(stderr, "lem2zip: %s: disk read failed\n", infile);
				unlink(temp_name);
    			return EXIT_FAILURE;
			}
			
			if (!compress_chunk(&chunk, d_buffer, d_buffer_size)) {
				fprintf(stderr, "lem2zip: %s: compression failed (this should never happen)\n", infile);
				unlink(temp_name);
    			return EXIT_FAILURE;
			}
			
			d_size -= d_buffer_size;
			
			chunk.flags = d_size == 0 ? 0xff : 0x00;
			
			if (!write_chunk(fo, &chunk)) {
				free(chunk.ref);
				free(chunk.data);
				break;
			}
			
			free(chunk.ref);
			free(chunk.data);
			
			assert(d_size >= 0);
			if (d_size == 0)
				break;
		}
		
		if (!feof(fi)) {
			int temp = ftell(fi);
			fseek(fi, 0, SEEK_END);
			if (temp - ftell(fi) > 0) {
				fprintf(stderr, "lem2zip: %s: unexpected termination of compression\n", infile);
				unlink(temp_name);
                return EXIT_FAILURE;
			}
		}
		
	}
	
	// success! replace the original
	fclose(fi); fi = NULL;
	fclose(fo); fo = NULL;
	
	unlink(infile);
	if (rename(temp_name, infile))
		fprintf(stderr, "lem2zip: %s could not be overwritten\n", infile);
	
	unlink(temp_name);
    return EXIT_SUCCESS;

}

/*
void usage( void )
{
    fprintf(stderr, "usage: %s [-dhLtV] [file ...]\n");
}

void version( void )
{
	fprintf(stderr, "%s %s (%s)\n", prog_ver, prog_date);
	fprintf(stderr, "Copyright (C) 2008 Carl Reinke\n");
	fprintf(stderr, "This is free software.  You may redistribute copies of it under the terms of\n"
	                "the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.\n"
	                "There is NO WARRANTY, to the extent permitted by law.\n");
}

void help( void )
{
    static char  *help_msg[] =
	{
		" -d --decompress  decompress",
		" -h --help        give this help",
		" -L --license     display software license",
		" -t --test        test compressed file integrity",
		" -V --version     display version number",
		" file ...         files to (de)compress",
		0
	};
    char **temp = help_msg;

    fprintf(stderr, "%s %s (%s)\n", prog_ver, prog_date);
    usage();
    while (*temp)
		fprintf(stderr, "%s\n", *temp++);
*/

// kate: tab-width 4;
